package com.FranckBarbier._Java._Railcar_control_system;

@javax.ejb.Singleton
@javax.ejb.Startup
public class Control_center implements Control_center_local, Control_center_remote {

    public final static String Message_separator = "Franck Barbier";
    /**
     * UML topography events
     */
    public final static String Alert100 = "alert100";
    public final static String Alert80 = "alert80";
    public final static String Terminal_crossing = "terminal crossing";
    public final static String Terminal_stopping = "terminal stopping";
    /**
     * JNDI
     */
    private javax.naming.Context _jndi_context;
    /**
     * JPA
     */
    @javax.persistence.PersistenceContext(unitName = "Railcar_control_system_database_PU")
    private javax.persistence.EntityManager _entity_manager = null;
    /**
     * UML
     */
    private java.util.Map<String, Railcar> _railcars = new java.util.HashMap<String, Railcar>();
    private java.util.Map<String, Terminal> _terminals = new java.util.HashMap<String, Terminal>();

    @javax.annotation.PostConstruct
    public void create() {
        assert (_entity_manager != null); // Dependency injection has succeeded!

        try {
            _jndi_context = new javax.naming.InitialContext();

            for (java.util.ListIterator<Railcar_POJO> i = _entity_manager.createNamedQuery("Railcar.findAll").getResultList().listIterator(); i.hasNext();) {
                Railcar_POJO r1 = i.next();
                Railcar r2 = (Railcar) _jndi_context.lookup("java:global/Railcar_control_system_Java_EE_6/RailcarBean!com.FranckBarbier._Java._Railcar_control_system.Railcar");
                r2.start(r1.get_railcar());
                _railcars.put(r1.get_railcar(), r2);
            }

            for (java.util.ListIterator<Terminal_POJO> i = _entity_manager.createNamedQuery("Terminal.findAll").getResultList().listIterator(); i.hasNext();) {
                Terminal_POJO t1 = i.next();
                Terminal t2 = (Terminal) _jndi_context.lookup("java:global/Railcar_control_system_Java_EE_6/TerminalBean!com.FranckBarbier._Java._Railcar_control_system.Terminal");
                t2.start(t1.get_terminal());
                _terminals.put(t1.get_terminal(), t2);
            }
            /**
             * The resulting order of terminals is the row-by-row database
             * order, which may not be the appropriate order, i.e., the
             * '_terminals' map must be reordered according to the adjacency of
             * terminals
             */
        } catch (Exception e) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.SEVERE, null, e);
        }
    }

    @javax.annotation.PreDestroy
    public void destroy() {
        System.out.println(Control_center.class.getName() + " destroy...");
        try {
            // Container-managed 'EntityManager' objects must not be closed by users:
            // _entity_manager.close();
            if (_jndi_context != null) {
                _jndi_context.close();
            }
        } catch (Exception e) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.SEVERE, null, e);
        }
    }

    @Override
    public int number_of_terminals() {
        return _terminals.size();
    }

    @Override
    public int number_of_railcars() {
        return _railcars.size();
    }

    @Override
    public String get_first_terminal() {
        assert (!_terminals.isEmpty());
        return _terminals.keySet().iterator().next();
    }

    @Override
    public boolean is_terminal_in_state(String terminal, String state) {
        return _terminals.get(terminal).in_state(state);
    }

    @Override
    public boolean terminal_destination_board_is_empty(String terminal) {
        return _terminals.get(terminal).destination_board_is_empty();
    }

    @Override
    public String[] get_railcars(String[] railcars) {
        return _railcars.keySet().toArray(railcars);
    }

    @Override
    public boolean get_is_clockwise(String railcar) {
        Railcar_POJO rp = _entity_manager.find(Railcar_POJO.class, railcar);
        assert (rp != null);
        return rp.getIsClockwise() == 0 ? false : true;
    }

    @Override
    public String get_origin_terminal(String railcar) {
        Railcar_POJO rp = _entity_manager.find(Railcar_POJO.class, railcar);
        assert (rp != null);
        return rp.getOriginTerminal().get_terminal();
    }

    @Override
    public String get_end_terminal(String railcar) {
        Railcar_POJO rp = _entity_manager.find(Railcar_POJO.class, railcar);
        assert (rp != null);
        return rp.getEndTerminal().get_terminal();
    }

    @Override
    public String get_clockwise_next(String terminal) {
        Terminal_POJO tp = _entity_manager.find(Terminal_POJO.class, terminal);
        assert (tp != null);
        return tp.get_clockwise_next();
    }

    @Override
    public String get_counterclockwise_next(String terminal) {
        Terminal_POJO tp = _entity_manager.find(Terminal_POJO.class, terminal);
        assert (tp != null);
        return tp.get_counterclockwise_next();
    }

    @Override
    public boolean is_railcar_in_state(String railcar, String state) {
        return _railcars.get(railcar).in_state(state);
    }

    @Override
    public boolean railcar_destination_board_is_empty(String railcar) {
        return _railcars.get(railcar).destination_board_is_empty();
    }

    @Override
    public void alert100(String terminal, String railcar, boolean is_clockwise) {
        System.out.println(Control_center.class.getName() + " alert100: to " + terminal + "<-" + railcar + "/" + (is_clockwise ? "clockwise" : "counterclockwise"));
        Railcar r = _railcars.get(railcar);

        assert (r != null && _entity_manager.find(Railcar_POJO.class, railcar) != null);
        assert ((_entity_manager.find(Railcar_POJO.class, railcar).getIsClockwise() == 0 ? false : true) == is_clockwise);

        Terminal t = _terminals.get(terminal);

        assert (t != null && _entity_manager.find(Terminal_POJO.class, terminal) != null);

        // UML Sequence Diagram:
        try {
            r.alert100(t, r, is_clockwise);
            t.alert100(t, r, is_clockwise);
        } catch (com.pauware.pauware_engine._Exception.Statechart_exception se) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.SEVERE, null, se);
            // Caution: JMS re-emission is missed!
        }
    }

    @Override
    public void alert80(String terminal, String railcar, boolean is_clockwise) {
        System.out.println(Control_center.class.getName() + " alert80: to " + terminal + "<-" + railcar + "/" + (is_clockwise ? "clockwise" : "counterclockwise"));
        Railcar r = _railcars.get(railcar);

        assert (r != null && _entity_manager.find(Railcar_POJO.class, railcar) != null);
        assert ((_entity_manager.find(Railcar_POJO.class, railcar).getIsClockwise() == 0 ? false : true) == is_clockwise);

        Terminal t = _terminals.get(terminal);

        assert (t != null && _entity_manager.find(Terminal_POJO.class, terminal) != null);

        // UML Sequence Diagram:
        try {
            r.alert80(t, r, is_clockwise);
        } catch (com.pauware.pauware_engine._Exception.Statechart_exception se) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.SEVERE, null, se);
            // Caution: JMS re-emission is missed!
        }
    }

    @Override
    public void railcar_new_destination(String railcar, String terminal) {
        System.out.println(Control_center.class.getName() + " new destination: in " + railcar + ", one intends to stop at " + terminal);
        Railcar r = _railcars.get(railcar);

        assert (r != null && _entity_manager.find(Railcar_POJO.class, railcar) != null);

        // UML Communication Diagram:
        try {
            r.new_destination(terminal);
        } catch (com.pauware.pauware_engine._Exception.Statechart_exception se) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.WARNING, null, se);
        }
    }

    @Override
    public void terminal_new_destination(String terminal, String another_terminal) {
        System.out.println(Control_center.class.getName() + " new destination: in " + terminal + " one intends to stop at " + another_terminal);
        Terminal t = _terminals.get(terminal);

        assert (t != null && _entity_manager.find(Terminal_POJO.class, terminal) != null);

        // UML Communication Diagram:
        try {
            t.new_destination(another_terminal);
        } catch (com.pauware.pauware_engine._Exception.Statechart_exception se) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.WARNING, null, se);
        }
    }

    @Override
    public void terminal_crossing(String terminal, String railcar, boolean is_clockwise) {
        System.out.println(Control_center.class.getName() + " terminal crossing: to " + terminal + "<-" + railcar + "/" + (is_clockwise ? "clockwise" : "counterclockwise"));
        Railcar r = _railcars.get(railcar);
        Railcar_POJO rp = _entity_manager.find(Railcar_POJO.class, railcar);

        assert (r != null && rp != null);
        assert ((rp.getIsClockwise() == 0 ? false : true) == is_clockwise);

        Terminal t = _terminals.get(terminal);
        Terminal_POJO tp = _entity_manager.find(Terminal_POJO.class, terminal);

        assert (t != null && tp != null);

        // UML Sequence Diagram:
        try {
            t.terminal_crossing(t, r, is_clockwise);
            // Database update:
            assert (is_clockwise ? tp.equals(rp.getEndTerminal()) : tp.equals(rp.getOriginTerminal()));
            if (is_clockwise) {
                rp.setOriginTerminal(tp);
                rp.setEndTerminal(_entity_manager.find(Terminal_POJO.class, tp.get_clockwise_next()));
            } else {
                rp.setOriginTerminal(_entity_manager.find(Terminal_POJO.class, tp.get_counterclockwise_next()));
                rp.setEndTerminal(tp);
            }
            _entity_manager.persist(rp);
        } catch (com.pauware.pauware_engine._Exception.Statechart_exception se) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.SEVERE, null, se);
            // Caution: JMS re-emission is missed!
        }
    }

    @Override
    public void terminal_stopping(String terminal, String railcar, boolean is_clockwise) {
        System.out.println(Control_center.class.getName() + " terminal stopping: to " + terminal + "<-" + railcar + "/" + (is_clockwise ? "clockwise" : "counterclockwise"));
        Railcar r = _railcars.get(railcar);
        Railcar_POJO rp = _entity_manager.find(Railcar_POJO.class, railcar);

        assert (r != null && rp != null);
        assert ((rp.getIsClockwise() == 0 ? false : true) == is_clockwise);

        Terminal t = _terminals.get(terminal);
        Terminal_POJO tp = _entity_manager.find(Terminal_POJO.class, terminal);

        assert (t != null && tp != null);
        assert (is_clockwise ? tp.equals(rp.getEndTerminal()) : tp.equals(rp.getOriginTerminal()));

        // UML Sequence Diagram:
        try {
            t.terminal_stopping(t, r, is_clockwise);
            // Database update:
            assert (is_clockwise ? tp.equals(rp.getEndTerminal()) : tp.equals(rp.getOriginTerminal()));
            if (is_clockwise) {
                rp.setOriginTerminal(tp);
                rp.setEndTerminal(_entity_manager.find(Terminal_POJO.class, tp.get_clockwise_next()));
            } else {
                rp.setOriginTerminal(_entity_manager.find(Terminal_POJO.class, tp.get_counterclockwise_next()));
                rp.setEndTerminal(tp);
            }
            _entity_manager.persist(rp);
        } catch (com.pauware.pauware_engine._Exception.Statechart_exception se) {
            java.util.logging.Logger.getLogger(Control_center.class.getName()).log(java.util.logging.Level.SEVERE, null, se);
            // Caution: JMS re-emission is missed!
        }
    }
}
