package com.FranckBarbier._Java._Railcar_control_system;

import com.pauware.pauware_engine._Core.*;
import com.pauware.pauware_engine._Exception.*;
import com.pauware.pauware_engine._Java_EE.*;

@javax.ejb.Stateful
// Transactions' inhibition:
@javax.ejb.TransactionManagement(javax.ejb.TransactionManagementType.BEAN)
public class RailcarBean implements Railcar {

    /**
     * EJB
     */
    @javax.annotation.Resource
    private javax.ejb.SessionContext _ejb_context = null;

    @Override
    public String get_name() {
        return (_Railcar_state_machine == null) ? RailcarBean.class.getName() : _Railcar_state_machine.name();
    }

    @Override
    public boolean destination_board_is_empty() {
        return _destination_board.isEmpty();
    }

    @javax.annotation.PostConstruct
    public void create() {
        try {
            init_structure();
            init_behavior();
        } catch (Statechart_exception se) {
            java.util.logging.Logger.getLogger(get_name()).log(java.util.logging.Level.SEVERE, null, se);
        }
        assert (_ejb_context != null);
    }

    @javax.annotation.PreDestroy
    public void destroy() {
        System.out.println(get_name() + " destroyed...");
        try {
            _Railcar_state_machine.stop();
            _cruiser.stop();
        } catch (Statechart_exception se) {
            java.util.logging.Logger.getLogger(RailcarBean.class.getName()).log(java.util.logging.Level.SEVERE, null, se);
        }
    }

    @javax.ejb.PrePassivate
    public void passivate() {
        System.out.println(get_name() + " passivated...");
    }

    @javax.ejb.PostActivate
    public void activate() {
        System.out.println(get_name() + " activated...");
    }
    /**
     * UML associations
     */
    private Cruiser _cruiser;
    private java.util.HashSet<String> _destination_board;

    /**
     * Simulation
     */
    @Override
    public boolean has_for_destination(String terminal) {
        return _destination_board.contains(terminal);
    }
    /**
     * UML statechart
     */
    private AbstractStatechart _Stopped_on_railway;
    private AbstractStatechart _Wait_for_terminal_order;
    private AbstractStatechart _Wait_for_alert80;
    private AbstractStatechart _Arriving;
    private AbstractStatechart _Cruising;
    private AbstractStatechart _Stopped_at_terminal;
    private AbstractStatechart_monitor _Railcar_state_machine;
    /**
     * UML state machine variables
     */
    private Terminal _my_next_possible_stop = null;
    private String _my_next_possible_stop_name = null;

    /**
     * UML creation
     */
    private void init_structure() throws Statechart_exception {
        _cruiser = new Cruiser();
        _destination_board = new java.util.HashSet<String>();
    }

    private void init_behavior() throws Statechart_exception {
        _Stopped_on_railway = new Statechart(Railcar.Stopped_on_railway);
        _Wait_for_terminal_order = new Statechart(Railcar.Wait_for_terminal_order);
        _Wait_for_alert80 = new Statechart(Railcar.Wait_for_alert80).set_entryAction(_cruiser, "set_Engaged20");
        _Arriving = (_Stopped_on_railway.xor(_Wait_for_terminal_order.xor(_Wait_for_alert80))).name(Railcar.Arriving);
        _Arriving.stateInvariant(this, "not_my_next_possible_stop_oclIsUndefined");
        _Cruising = new Statechart(Railcar.Cruising).set_entryAction(_cruiser, "set_Engaged80");
        _Cruising.inputState();
        _Stopped_at_terminal = new Statechart(Railcar.Stopped_at_terminal).set_exitAction(_cruiser, "set_Engaged20");
        _Railcar_state_machine = new Statechart_monitor((_Arriving.xor(_Cruising)).xor(_Stopped_at_terminal), Railcar.class.getName(), AbstractStatechart_monitor.Show_on_system_out);
    }

    @Override
    public void start(String railcar) throws Statechart_exception {
        _Railcar_state_machine.name(railcar);
        _Railcar_state_machine.fires(Control_center.Alert100, _Cruising, _Wait_for_alert80);
        _Railcar_state_machine.fires("go", _Stopped_at_terminal, _Cruising);
        _Railcar_state_machine.fires("go on", _Stopped_on_railway, _Wait_for_terminal_order, true, _cruiser, "set_Engaged20");
        _Railcar_state_machine.fires("stop", _Wait_for_terminal_order, _Stopped_on_railway, true, _cruiser, "set_Disengaged");
        _Railcar_state_machine.fires(Control_center.Terminal_crossing, _Wait_for_terminal_order, _Cruising);
        _Railcar_state_machine.fires(Control_center.Terminal_stopping, _Wait_for_terminal_order, _Stopped_at_terminal, true, _cruiser, "set_Disengaged");
        _cruiser.start(_Railcar_state_machine.name());
        _Railcar_state_machine.start();
    }

    /**
     * UML events
     */
    @Override
    public void alert100(Terminal terminal, Railcar railcar, Boolean is_clockwise) throws Statechart_exception {
        /**
         * UML pre-condition(s)
         */
        /* Downcasting is not satisfactory here: */ assert (_ejb_context.getEJBLocalObject().isIdentical((javax.ejb.EJBLocalObject) railcar));
        /**
         * UML post-condition(s)
         */
        _my_next_possible_stop = terminal;
        _my_next_possible_stop_name = _my_next_possible_stop.get_name();
        _Railcar_state_machine.run_to_completion(Control_center.Alert100);
    }

    @Override
    public void alert80(Terminal terminal, Railcar railcar, Boolean is_clockwise) throws Statechart_exception {
        /**
         * UML pre-condition(s)
         */
        /* Downcasting is not satisfactory here: */ assert (_ejb_context.getEJBLocalObject().isIdentical((javax.ejb.EJBLocalObject) railcar));
        assert (_my_next_possible_stop != null); // Checked through a state invariant on '_Arriving'
        /* Downcasting is not satisfactory here: */ assert (((javax.ejb.EJBLocalObject) _my_next_possible_stop).isIdentical((javax.ejb.EJBLocalObject) terminal));
        // The 'approaching' event is sent in 'Reentrance' mode since 'stop' events may be (fortuitously) received through loopback calls
        _Wait_for_terminal_order.reset_entryAction(_my_next_possible_stop, "approaching", new Object[]{terminal, railcar, is_clockwise}, AbstractStatechart_monitor.Reentrance);
        _Railcar_state_machine.fires(Control_center.Alert80, _Wait_for_alert80, _Wait_for_terminal_order);
        _Railcar_state_machine.run_to_completion(Control_center.Alert80);
    }

    @Override
    public void go() throws Statechart_exception {
        _Railcar_state_machine.run_to_completion("go");
    }

    @Override
    public void go_on() throws Statechart_exception {
        _Railcar_state_machine.run_to_completion("go on");
    }

    @Override
    public void new_destination(String another_terminal) throws Statechart_exception {
        _Railcar_state_machine.allowedEvent("new destination", this, "new_destination_UML_post_conditions", new Object[]{another_terminal});
        _Railcar_state_machine.run_to_completion("new destination");
    }

    @Override
    public void stop() throws Statechart_exception {
        _Railcar_state_machine.run_to_completion("stop");
    }

    @Override
    public void terminal_crossing() throws Statechart_exception {
        /**
         * UML pre-condition(s)
         */
        assert (!_destination_board.contains(_my_next_possible_stop_name));
        /**
         * UML post-condition(s)
         */
        _my_next_possible_stop = null;
        _my_next_possible_stop_name = null;
        _Railcar_state_machine.run_to_completion(Control_center.Terminal_crossing);
    }

    @Override
    public void terminal_stopping() throws Statechart_exception {
        /**
         * UML post-condition(s)
         */
        _destination_board.remove(_my_next_possible_stop_name);
        _my_next_possible_stop = null;
        _my_next_possible_stop_name = null;
        _Railcar_state_machine.run_to_completion(Control_center.Terminal_stopping);
    }

    /**
     * UML invariants
     */
    public boolean not_my_next_possible_stop_oclIsUndefined() {
        return _my_next_possible_stop != null;
    }

    /**
     * UML post-conditions
     */
    public void new_destination_UML_post_conditions(String another_terminal) {
        _destination_board.add(another_terminal);
    }

    /**
     * Management
     */
    @Override
    public String async_current_state() {
        return _Railcar_state_machine.async_current_state();
    }

    @Override
    public String current_state() {
        return _Railcar_state_machine.current_state();
    }

    @Override
    public boolean in_state(String name) {
        return _Railcar_state_machine.in_state(name);
    }

    @Override
    public String name() {
        return _Railcar_state_machine.name();
    }

    @Override
    public String verbose() {
        return _Railcar_state_machine.verbose();
    }
}
